@@ -1,10 +1,12 @@
 #!/bin/sh 
-if [ $# -lt 3 ]; then
-	echo "Need 3 arguments" > err.log
+if [ $# -lt 4 ]; then
+	echo "Need 4 arguments" > err.log
 	exit 1
 fi
 
 Rscript=$1
+R_HOME=$4
+
 if [ ! -r $Rscript ]; then
 	echo $Rscript "does not exist or is not readable!" > err.log
 	exit 1
@@ -12,7 +14,7 @@ fi
 
 if [ $3 == "needlog" ]; then
 	hn=`hostname -s`
-	R --no-init-file --slave --no-save < $1 > $hn.$2.$$.log
+	$R_HOME/bin/R --no-init-file --slave --no-save < $1 > $hn.$2.$$.log
 else
-	R --no-init-file --slave --no-save < $1 > /dev/null
+	$R_HOME/bin/R --no-init-file --slave --no-save < $1 > /dev/null
 fi
